"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const crypto = require("crypto");
const rimraf = require("rimraf");
const format_1 = require("./format");
const promise_1 = require("./promise");
const modules_1 = require("../modules");
exports.ERROR_FILE_NOT_FOUND = 'FILE_NOT_FOUND';
exports.ERROR_FILE_INVALID_JSON = 'FILE_INVALID_JSON';
exports.ERROR_OVERWRITE_DENIED = 'OVERWRITE_DENIED';
exports.fsMkdir = promise_1.promisify(fs.mkdir);
exports.fsStat = promise_1.promisify(fs.stat);
exports.fsReadFile = promise_1.promisify(fs.readFile);
exports.fsWriteFile = promise_1.promisify(fs.writeFile);
exports.fsReadDir = promise_1.promisify(fs.readdir);
exports.rimrafp = promise_1.promisify(rimraf);
function readDir(filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield exports.fsReadDir(filePath);
        }
        catch (e) {
            if (e.code === 'ENOENT') {
                return [];
            }
            throw e;
        }
    });
}
exports.readDir = readDir;
function fsReadJsonFile(filePath, options = { encoding: 'utf8' }) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const f = yield exports.fsReadFile(filePath, options);
            return JSON.parse(f);
        }
        catch (e) {
            if (e.code === 'ENOENT') {
                throw exports.ERROR_FILE_NOT_FOUND;
            }
            else if (e instanceof SyntaxError) {
                throw exports.ERROR_FILE_INVALID_JSON;
            }
            throw e;
        }
    });
}
exports.fsReadJsonFile = fsReadJsonFile;
function fsWriteJsonFile(filePath, json, options) {
    return __awaiter(this, void 0, void 0, function* () {
        return exports.fsWriteFile(filePath, JSON.stringify(json, null, 2) + '\n', options);
    });
}
exports.fsWriteJsonFile = fsWriteJsonFile;
function fileToString(filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield exports.fsReadFile(filepath, { encoding: 'utf8' });
        }
        catch (e) {
            if (e.code === 'ENOENT') {
                return '';
            }
            throw e;
        }
    });
}
exports.fileToString = fileToString;
function permissionToOverwrite(p) {
    return __awaiter(this, void 0, void 0, function* () {
        let stats;
        try {
            stats = yield exports.fsStat(p);
        }
        catch (e) {
            if (e.code !== 'ENOENT') {
                throw e;
            }
            return true;
        }
        if (stats.isFile()) {
            const inquirer = modules_1.load('inquirer');
            const confirmation = yield inquirer.prompt({
                type: 'confirm',
                name: 'apply',
                message: `File exists: '${format_1.prettyPath(p)}'. Overwrite?`
            });
            if (!confirmation['apply']) {
                return false;
            }
        }
        return true;
    });
}
exports.permissionToOverwrite = permissionToOverwrite;
function fsWriteFilePromptOverwrite(p, data, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!(yield permissionToOverwrite(p))) {
            throw exports.ERROR_OVERWRITE_DENIED;
        }
        return exports.fsWriteFile(p, data, options);
    });
}
exports.fsWriteFilePromptOverwrite = fsWriteFilePromptOverwrite;
function fsMkdirp(p, mode) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof mode === 'undefined') {
            mode = 0o777 & (~process.umask());
        }
        const absPath = path.resolve(p);
        const pathObj = path.parse(absPath);
        const dirnames = absPath.split(path.sep).splice(1);
        const dirs = dirnames.map((v, i) => path.resolve(pathObj.root, ...dirnames.slice(0, i), v));
        for (let dir of dirs) {
            try {
                yield exports.fsMkdir(dir, mode);
            }
            catch (e) {
                if (e.code !== 'EEXIST') {
                    throw e;
                }
            }
        }
    });
}
exports.fsMkdirp = fsMkdirp;
function getFileChecksum(filePath) {
    return new Promise((resolve, reject) => {
        const hash = crypto.createHash('md5');
        const input = fs.createReadStream(filePath);
        input.on('error', (err) => {
            reject(err);
        });
        hash.once('readable', () => {
            const fullChecksum = hash.read().toString('hex');
            resolve(fullChecksum);
        });
        input.pipe(hash);
    });
}
exports.getFileChecksum = getFileChecksum;
function writeStreamToFile(stream, destination) {
    return new Promise((resolve, reject) => {
        const dest = fs.createWriteStream(destination);
        stream.pipe(dest);
        dest.on('error', reject);
        dest.on('finish', resolve);
    });
}
exports.writeStreamToFile = writeStreamToFile;
function copyDirectory(source, destionation) {
    return new Promise((resolve, reject) => {
        const ncp = modules_1.load('ncp');
        ncp.ncp(source, destionation, (err) => {
            if (err) {
                reject(err);
            }
            resolve();
        });
    });
}
exports.copyDirectory = copyDirectory;
function copyFile(fileName, target, mode = 0o777) {
    return new Promise((resolve, reject) => {
        let readStream = fs.createReadStream(fileName);
        let writeStream = fs.createWriteStream(target, { mode: mode });
        readStream.on('error', reject);
        writeStream.on('error', reject);
        writeStream.on('open', function () {
            readStream.pipe(writeStream);
        });
        writeStream.once('finish', resolve);
    });
}
exports.copyFile = copyFile;
